import json
import decimal
import datetime
import base64


class LazyEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, decimal.Decimal):
            return float(obj)
        elif isinstance(obj, datetime.datetime):
            # # 将datetime对象转换为ISO格式字符串
            return obj.isoformat()
        elif isinstance(obj, datetime.date):
            return obj.isoformat()
        elif isinstance(obj, bytes):
            # 将bytes转换为base64编码的字符串
            try:
                # 尝试将bytes解码为UTF-8字符串
                return obj.decode('utf-8')
            except UnicodeDecodeError:
                # 如果UTF-8解码失败，回退到Base64编码
                return base64.b64encode(obj).decode('utf-8')
        # elif isinstance(obj, bson.timestamp.Timestamp):
        #     # # 将BSON时间戳转换为其时间表示的ISO格式字符串
        #     return obj.as_datetime().isoformat()
        # elif isinstance(obj, bson.objectid.ObjectId):
        #     # # 将ObjectId转换为字符串
        #     return str(obj)
        return super(LazyEncoder, self).default(obj)


def json2str(
        data: json,
        ensure_ascii: bool = False
):
    """
    在将json数据反序列化为str时，会遇到一些格式无法转换
    这里使用识别类型转换转为str
    目前支持类型：

    对于mongodb返回数据的处理：
        from bson import json_util
        default=json_util.default

    decimal --> str
    datetime.datetime --> str(%Y-%m-%d %H:%M:%S)
    datetime.date --> str(%Y-%m-%d)
    """
    return json.dumps(
        data,
        cls=LazyEncoder,
        ensure_ascii=ensure_ascii
    )



import os
import platform

def system_beep():
    system = platform.system()
    if system == "Windows":
        import winsound
        winsound.Beep(1000, 1000)
    elif system == "Darwin":  # macOS
        os.system('say "警报"')
        # 或者播放系统声音
        os.system('afplay /System/Library/Sounds/Ping.aiff')
    elif system == "Linux":
        # 需要安装sox: sudo apt-get install sox
        os.system('play -nq -t alsa synth 1 sine 1000')
    else:
        print("\a")  # 终端响铃

# 使用
system_beep()


import platform
import sys
import re
import subprocess
import os
import zipfile
import tarfile
from lazysdk import lazyfile

# from webdriver_manager.core.utils import linux_browser_apps_to_cmd, windows_browser_apps_to_cmd, \
#     read_version_from_cmd

"""
借鉴webdriver_manager==4.0.1代码，但是原模块不好用，所以这里精简了
"""
ROOT_FOLDER_NAME = ".lazy_chromedriver"
DEFAULT_PROJECT_ROOT_CACHE_PATH = os.path.join(sys.path[0], ROOT_FOLDER_NAME)
DEFAULT_USER_HOME_CACHE_PATH = os.path.join(os.path.expanduser("~"), ROOT_FOLDER_NAME)
drivers_root = "drivers"
drivers_directory = os.path.join(DEFAULT_USER_HOME_CACHE_PATH, drivers_root)


def determine_powershell():
    """Returns "True" if runs in Powershell and "False" if another console."""
    cmd = "(dir 2>&1 *`|echo CMD);&<# rem #>echo powershell"
    with subprocess.Popen(
            cmd,
            stdout=subprocess.PIPE,
            stderr=subprocess.DEVNULL,
            stdin=subprocess.DEVNULL,
            shell=True,
    ) as stream:
        stdout = stream.communicate()[0].decode()
    return "" if stdout == "powershell" else "powershell"


def linux_browser_apps_to_cmd(*apps: str) -> str:
    """Create 'browser --version' command from browser app names.

    Result command example:
        chromium --version || chromium-browser --version
    """
    ignore_errors_cmd_part = " 2>/dev/null" if os.getenv(
        "WDM_LOG_LEVEL") == "0" else ""
    return " || ".join(f"{i} --version{ignore_errors_cmd_part}" for i in apps)


def windows_browser_apps_to_cmd(*apps: str) -> str:
    """Create analogue of browser --version command for windows."""
    powershell = determine_powershell()

    first_hit_template = """$tmp = {expression}; if ($tmp) {{echo $tmp; Exit;}};"""
    script = "$ErrorActionPreference='silentlycontinue'; " + " ".join(
        first_hit_template.format(expression=e) for e in apps
    )

    return f'{powershell} -NoProfile "{script}"'


def read_version_from_cmd(cmd, pattern):
    with subprocess.Popen(
            cmd,
            stdout=subprocess.PIPE,
            stdin=subprocess.DEVNULL,
            shell=True,
    ) as stream:
        stdout = stream.communicate()[0].decode()
        version = re.search(pattern, stdout)
        version = version.group(0) if version else None
    return version


class ChromeType(object):
    GOOGLE = "google-chrome"
    CHROMIUM = "chromium"
    BRAVE = "brave-browser"
    MSEDGE = "edge"


class OSType(object):
    LINUX = "linux"
    MAC = "mac"
    WIN = "win"


PATTERN = {
    ChromeType.CHROMIUM: r"\d+\.\d+\.\d+",
    ChromeType.GOOGLE: r"\d+\.\d+\.\d+.\d+",  # 增加了最小版本
    ChromeType.MSEDGE: r"\d+\.\d+\.\d+",
    "brave-browser": r"\d+\.\d+\.\d+(\.\d+)?",
    "firefox": r"(\d+.\d+)",
}


def get_os_name():
    pl = sys.platform
    if pl == "linux" or pl == "linux2":
        return OSType.LINUX
    elif pl == "darwin":
        return OSType.MAC
    elif pl == "win32" or pl == "cygwin":
        return OSType.WIN


def get_os_type():
    pl = sys.platform
    if pl == "darwin":
        return f'mac-{platform.machine()}'
    elif pl == "win32" or pl == "cygwin":
        if platform.machine().endswith("64"):
            return 'win64'
        else:
            return 'win32'


class OperationSystemManager(object):

    def __init__(self, os_type=None):
        self._os_type = os_type

    @staticmethod
    def get_os_name():
        pl = sys.platform
        if pl == "linux" or pl == "linux2":
            return OSType.LINUX
        elif pl == "darwin":
            return OSType.MAC
        elif pl == "win32" or pl == "cygwin":
            return OSType.WIN

    @staticmethod
    def get_os_architecture():
        if platform.machine().endswith("64"):
            return 64
        else:
            return 32

    def get_os_type(self):
        if self._os_type:
            return self._os_type
        return f"{self.get_os_name()}{self.get_os_architecture()}"

    @staticmethod
    def is_arch(os_sys_type):
        if '_m1' in os_sys_type:
            return True
        return platform.processor() != 'i386'

    @staticmethod
    def is_mac_os(os_sys_type):
        return OSType.MAC in os_sys_type

    def get_browser_version_from_os(self, browser_type=None):
        """Return installed browser version."""
        cmd_mapping = {
            ChromeType.GOOGLE: {
                OSType.LINUX: linux_browser_apps_to_cmd(
                    "google-chrome",
                    "google-chrome-stable",
                    "google-chrome-beta",
                    "google-chrome-dev",
                ),
                OSType.MAC: r"/Applications/Google\ Chrome.app/Contents/MacOS/Google\ Chrome --version",
                OSType.WIN: windows_browser_apps_to_cmd(
                    r'(Get-Item -Path "$env:PROGRAMFILES\Google\Chrome\Application\chrome.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:PROGRAMFILES (x86)\Google\Chrome\Application\chrome.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:LOCALAPPDATA\Google\Chrome\Application\chrome.exe").VersionInfo.FileVersion',
                    r'(Get-ItemProperty -Path Registry::"HKCU\SOFTWARE\Google\Chrome\BLBeacon").version',
                    r'(Get-ItemProperty -Path Registry::"HKLM\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\Google Chrome").version',
                ),
            },
            ChromeType.CHROMIUM: {
                OSType.LINUX: linux_browser_apps_to_cmd("chromium", "chromium-browser"),
                OSType.MAC: r"/Applications/Chromium.app/Contents/MacOS/Chromium --version",
                OSType.WIN: windows_browser_apps_to_cmd(
                    r'(Get-Item -Path "$env:PROGRAMFILES\Chromium\Application\chrome.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:PROGRAMFILES (x86)\Chromium\Application\chrome.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:LOCALAPPDATA\Chromium\Application\chrome.exe").VersionInfo.FileVersion',
                    r'(Get-ItemProperty -Path Registry::"HKCU\SOFTWARE\Chromium\BLBeacon").version',
                    r'(Get-ItemProperty -Path Registry::"HKLM\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\Chromium").version',
                ),
            },
            ChromeType.BRAVE: {
                OSType.LINUX: linux_browser_apps_to_cmd(
                    "brave-browser", "brave-browser-beta", "brave-browser-nightly"
                ),
                OSType.MAC: r"/Applications/Brave\ Browser.app/Contents/MacOS/Brave\ Browser --version",
                OSType.WIN: windows_browser_apps_to_cmd(
                    r'(Get-Item -Path "$env:PROGRAMFILES\BraveSoftware\Brave-Browser\Application\brave.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:PROGRAMFILES (x86)\BraveSoftware\Brave-Browser\Application\brave.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:LOCALAPPDATA\BraveSoftware\Brave-Browser\Application\brave.exe").VersionInfo.FileVersion',
                    r'(Get-ItemProperty -Path Registry::"HKCU\SOFTWARE\BraveSoftware\Brave-Browser\BLBeacon").version',
                    r'(Get-ItemProperty -Path Registry::"HKLM\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\BraveSoftware Brave-Browser").version',
                ),
            },
            ChromeType.MSEDGE: {
                OSType.LINUX: linux_browser_apps_to_cmd(
                    "microsoft-edge",
                    "microsoft-edge-stable",
                    "microsoft-edge-beta",
                    "microsoft-edge-dev",
                ),
                OSType.MAC: r"/Applications/Microsoft\ Edge.app/Contents/MacOS/Microsoft\ Edge --version",
                OSType.WIN: windows_browser_apps_to_cmd(
                    # stable edge
                    r'(Get-Item -Path "$env:PROGRAMFILES\Microsoft\Edge\Application\msedge.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:PROGRAMFILES (x86)\Microsoft\Edge\Application\msedge.exe").VersionInfo.FileVersion',
                    r'(Get-ItemProperty -Path Registry::"HKCU\SOFTWARE\Microsoft\Edge\BLBeacon").version',
                    r'(Get-ItemProperty -Path Registry::"HKLM\SOFTWARE\Microsoft\EdgeUpdate\Clients\{56EB18F8-8008-4CBD-B6D2-8C97FE7E9062}").pv',
                    # beta edge
                    r'(Get-Item -Path "$env:LOCALAPPDATA\Microsoft\Edge Beta\Application\msedge.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:PROGRAMFILES\Microsoft\Edge Beta\Application\msedge.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:PROGRAMFILES (x86)\Microsoft\Edge Beta\Application\msedge.exe").VersionInfo.FileVersion',
                    r'(Get-ItemProperty -Path Registry::"HKCU\SOFTWARE\Microsoft\Edge Beta\BLBeacon").version',
                    # dev edge
                    r'(Get-Item -Path "$env:LOCALAPPDATA\Microsoft\Edge Dev\Application\msedge.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:PROGRAMFILES\Microsoft\Edge Dev\Application\msedge.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:PROGRAMFILES (x86)\Microsoft\Edge Dev\Application\msedge.exe").VersionInfo.FileVersion',
                    r'(Get-ItemProperty -Path Registry::"HKCU\SOFTWARE\Microsoft\Edge Dev\BLBeacon").version',
                    # canary edge
                    r'(Get-Item -Path "$env:LOCALAPPDATA\Microsoft\Edge SxS\Application\msedge.exe").VersionInfo.FileVersion',
                    r'(Get-ItemProperty -Path Registry::"HKCU\SOFTWARE\Microsoft\Edge SxS\BLBeacon").version',
                    # highest edge
                    r"(Get-Item (Get-ItemProperty 'HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\msedge.exe').'(Default)').VersionInfo.ProductVersion",
                    r"[System.Diagnostics.FileVersionInfo]::GetVersionInfo((Get-ItemProperty 'HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\msedge.exe').'(Default)').ProductVersion",
                    r"Get-AppxPackage -Name *MicrosoftEdge.* | Foreach Version",
                    r'(Get-ItemProperty -Path Registry::"HKLM\SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\Microsoft Edge").version',
                ),
            },
            "firefox": {
                OSType.LINUX: linux_browser_apps_to_cmd("firefox"),
                OSType.MAC: r"/Applications/Firefox.app/Contents/MacOS/firefox --version",
                OSType.WIN: windows_browser_apps_to_cmd(
                    r'(Get-Item -Path "$env:PROGRAMFILES\Mozilla Firefox\firefox.exe").VersionInfo.FileVersion',
                    r'(Get-Item -Path "$env:PROGRAMFILES (x86)\Mozilla Firefox\firefox.exe").VersionInfo.FileVersion',
                    r"(Get-Item (Get-ItemProperty 'HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\firefox.exe').'(Default)').VersionInfo.ProductVersion",
                    r'(Get-ItemProperty -Path Registry::"HKLM\SOFTWARE\Mozilla\Mozilla Firefox").CurrentVersion',
                ),
            },
        }

        try:
            cmd_mapping = cmd_mapping[browser_type][OperationSystemManager.get_os_name()]
            pattern = PATTERN[browser_type]
            version = read_version_from_cmd(cmd_mapping, pattern)
            return version
        except Exception:
            return None
            # raise Exception("Can not get browser version from OS")


def get_driver_download_url(
        driver_url='https://edgedl.me.gvt1.com/edgedl/chrome/chrome-for-testing',
):
    os_type = get_os_type()
    driver_version = OperationSystemManager().get_browser_version_from_os(browser_type='google-chrome')
    return f"{driver_url}/{driver_version}/{os_type}/chromedriver-{os_type}.zip"


# def save_archive_file(file: File, directory: str):
#     os.makedirs(directory, exist_ok=True)
#
#     archive_path = f"{directory}{os.sep}{file.filename}"
#     with open(archive_path, "wb") as code:
#         code.write(file.content)
#     if not os.path.exists(archive_path):
#         raise FileExistsError(f"No file has been saved on such path {archive_path}")
#     return Archive(archive_path)
#
#
def download_driver():
    """
    根据当前系统的chrome版本下载对应版本的driver
    """
    from lazysdk import lazyfile
    driver_download_url = get_driver_download_url()
    os.makedirs(drivers_directory, exist_ok=True)  # 创建driver下载目录
    filename = extract_filename_from_url(driver_download_url)
    print('filename:', filename)
    archive_path = os.path.join(drivers_directory, filename)
    # archive_path = f"{directory}{os.sep}{file.filename}"
    print(driver_download_url)
    file = lazyfile.download(
        url=driver_download_url,
        filename=filename,
        path=drivers_directory
    )['file_dir']
    return unpack_archive(archive_file=file, target_dir=drivers_directory)


class LinuxZipFileWithPermissions(zipfile.ZipFile):
    """Class for extract files in linux with right permissions"""

    def extract(self, member, path=None, pwd=None):
        if not isinstance(member, zipfile.ZipInfo):
            member = self.getinfo(member)

        if path is None:
            path = os.getcwd()

        ret_val = self._extract_member(member, path, pwd)  # noqa
        attr = member.external_attr >> 16
        os.chmod(ret_val, attr)
        return ret_val


def unpack_archive(archive_file, target_dir):
    if archive_file.endswith(".zip"):
        return lazyfile.unzip(file=archive_file)
    elif archive_file.endswith(".tar.gz"):
        return __extract_tar_file(archive_file, target_dir)


def __extract_zip(archive_file, to_directory):
    zip_class = zipfile.ZipFile
    archive = zip_class(archive_file)
    try:
        archive.extractall(to_directory)
    except Exception as e:
        if e.args[0] not in [26, 13] and e.args[1] not in [
            "Text file busy",
            "Permission denied",
        ]:
            raise e
        file_names = []
        for n in archive.namelist():
            if "/" not in n:
                file_names.append(n)
            else:
                file_path, file_name = n.split("/")
                full_file_path = os.path.join(to_directory, file_path)
                source = os.path.join(full_file_path, file_name)
                destination = os.path.join(to_directory, file_name)
                os.replace(source, destination)
                file_names.append(file_name)
        return sorted(file_names, key=lambda x: x.lower())
    return archive.namelist()


def __extract_tar_file(archive_file, to_directory):
    try:
        tar = tarfile.open(archive_file.file_path, mode="r:gz")
    except tarfile.ReadError:
        tar = tarfile.open(archive_file.file_path, mode="r:bz2")
    members = tar.getmembers()
    tar.extractall(to_directory)
    tar.close()
    return [x.name for x in members]


def extract_filename_from_url(url):
    """
    从url中获取文件名
    """
    # Split the URL by '/'
    url_parts = url.split('/')
    # Get the last part of the URL, which should be the filename
    filename = url_parts[-1]
    # Decode the URL-encoded filename
    filename = os.path.basename(filename)
    return filename



#!/usr/bin/env python3
# coding = utf8
"""
@ Author : ZeroSeeker
@ e-mail : zeroseeker@foxmail.com
@ GitHub : https://github.com/ZeroSeeker
@ Gitee : https://gitee.com/ZeroSeeker
"""
import time

from lazysdk import lazyrequests
import showlog
import json
import envx
from urllib.parse import urlparse

"""
企业微信文档：
    https://work.weixin.qq.com/api/doc/90000/90136/91770
    https://work.weixin.qq.com/help?person_id=1&doc_id=13376
    获取会话内容：https://work.weixin.qq.com/api/doc/90000/90135/91774
"""

env_file_name_default = 'webhook.env'
test_msg = 'This is a test message from lazywebhook. \nIf you see this message, the test was successful.'


def make_con_info(
        env_file_name: str = env_file_name_default
):
    # ---------------- 固定设置 ----------------
    inner_env = envx.read(file_name=env_file_name)
    if inner_env is None:
        return
    else:
        webhook = inner_env.get('webhook')
        if webhook is None:
            showlog.warning(f'环境文件[{env_file_name}]未设置webhook参数')
            return
        else:
            at_ids = inner_env.get('at_ids')
            if at_ids is not None:
                at_ids = at_ids.split(',')
            at_mobiles = inner_env.get('at_mobiles')
            if at_mobiles is not None:
                at_mobiles = at_mobiles.split(',')
            con_info = {
                "webhook": inner_env.get('webhook'),
                "at_ids": at_ids,  # 艾特的id，以英文逗号分隔
                "at_mobiles": at_mobiles  # 艾特的手机号码，以英文逗号分隔
            }
            return con_info
    # ---------------- 固定设置 ----------------


class WeixinBasics:
    def __init__(
            self,
            con_info: dict = None,
            webhook: str = None,
            at_ids: list = None,
            at_mobiles: list = None,
            is_at: bool = None,
            at_all: bool = None
    ):
        """
        初始化参数
        :param con_info: 参数字典，将每个参数以key-value形式存放在一起
        :param webhook: webhook地址
        :param at_ids: 按照id艾特的id列表
        :param at_mobiles: 按照手机号码艾特的手机号码列表
        :param is_at: 是否艾特
        :param at_all: 是否艾特所有人

        con_info为参数集合，如果其他参数未指定，将从con_info中获取参数
        如果at_all为True，将艾特所有人，不会另外单独艾特单独的用户
        如果需要单独艾特某些id，需设置is_at=True，否则不会生效

        """
        if webhook is None and con_info is not None:
            self.webhook = con_info.get('webhook')
        else:
            self.webhook = webhook

        if at_ids is None and con_info is not None:
            self.at_ids = con_info.get('at_ids')
        else:
            self.at_ids = at_ids

        if at_mobiles is None and con_info is not None:
            self.at_mobiles = con_info.get('at_mobiles')
        else:
            self.at_mobiles = at_mobiles

        if is_at is None and con_info is not None:
            self.is_at = con_info.get('is_at', False)
        else:
            self.is_at = is_at

        if at_all is None and con_info is not None:
            self.at_all = con_info.get('at_all', False)
        else:
            self.at_all = at_all

    def send_text(
            self,
            msg: str = None,

            con_info: dict = None,
            webhook: str = None,
            at_ids: list = None,
            at_mobiles: list = None,
            is_at: bool = None,
            at_all: bool = None
    ):
        """
        发送文本消息

        :param msg: 发送的文本内容
        :param con_info: 参数字典，将每个参数以key-value形式存放在一起
        :param webhook: webhook地址
        :param at_ids: 按照id艾特的id列表
        :param at_mobiles: 按照手机号码艾特的手机号码列表
        :param is_at: 是否艾特
        :param at_all: 是否艾特所有人

        正常返回：
            {'errcode': 0, 'errmsg': 'ok'}
        异常返回：
            未指定发送内容
                {'errcode': 44004, 'errmsg': 'empty content, hint: [1653883914340381947793508], from ip: 121.227.209.88, more info at https://open.work.weixin.qq.com/devtool/query?e=44004'}
            接口调用超过限制
                {'errcode': 45009, 'errmsg': 'api freq out of limit, hint: [1653710540075724015410106], from ip: 121.227.209.88, more info at https://open.work.weixin.qq.com/devtool/query?e=45009'}
            机器人webhookurl不合法或者机器人已经被移除出群
                {'errcode': 93000, 'errmsg': 'invalid webhook url, hint: [1653710655173452988930371], from ip: 121.227.209.88, more info at https://open.work.weixin.qq.com/devtool/query?e=93000'}
        """
        if msg is None:
            msg = test_msg

        if webhook is None and con_info is not None:
            webhook = con_info.get('webhook')
        else:
            if webhook is None:
                webhook = self.webhook

        if at_ids is None and con_info is not None:
            at_ids = con_info.get('at_ids')
        else:
            if at_ids is None:
                at_ids = self.at_ids

        if at_mobiles is None and con_info is not None:
            at_mobiles = con_info.get('at_mobiles')
        else:
            if at_mobiles is None:
                at_mobiles = self.at_mobiles

        if is_at is None and con_info is not None:
            is_at = con_info.get('is_at', False)
        else:
            if is_at is None:
                is_at = self.is_at

        if at_all is None and con_info is not None:
            at_all = con_info.get('at_all', False)
        else:
            if at_all is None:
                at_all = self.at_all

        if at_all is True:
            at_ids = ["@all"]
            at_mobiles = ["@all"]

        text_dict = {
            'content': msg
        }
        if at_all is True:
            text_dict['mentioned_list'] = ["@all"]
        else:
            if at_ids is not None and is_at is True:
                text_dict['mentioned_list'] = at_ids
            if at_mobiles is not None and is_at is True:
                text_dict['mentioned_mobile_list'] = at_mobiles
        data = {
            "msgtype": "text",
            "text": text_dict
        }
        inner_data = json.dumps(data, ensure_ascii=False)
        byte_data = inner_data.encode('utf-8')
        response = lazyrequests.lazy_requests(
            method='POST',
            url=webhook,
            data=byte_data,
            return_json=True
        )
        return response


def send_text(
        msg: str = None,

        env_file_name: str = None,
        webhook: str = None,
        at_ids: list = None,
        at_mobiles: list = None,
        is_at: bool = None,
        at_all: bool = None,
        ensure_success: bool = False,
        ensure_success_limit: int = 60
):
    """
    在内部实例化，
    首先使用自定义参数，其次使用env参数
    """
    if env_file_name is not None:
        env_con_info = make_con_info(env_file_name=env_file_name)
    else:
        env_con_info = make_con_info()

    if webhook is None and env_file_name is not None and env_con_info is None:
        showlog.warning('webhook不能为空')
        return {'errcode': -1, 'errmsg': 'webhook不能为空'}
    elif webhook is None and env_file_name is not None and env_con_info is not None:
        webhook = env_con_info.get('webhook')

    if at_ids is None and env_file_name is not None and env_con_info is not None:
        at_ids = env_con_info.get('at_ids')

    if at_mobiles is None and env_file_name is not None and env_con_info is not None:
        at_mobiles = env_con_info.get('at_mobiles')

    con_info = {
        'webhook': webhook,
        'at_ids': at_ids,
        'at_mobiles': at_mobiles
    }
    webhook_hostname = urlparse(webhook).hostname
    if webhook_hostname == 'qyapi.weixin.qq.com':
        webhook_basic = WeixinBasics(con_info=con_info)
        retry_count = 0
        while True:
            response = webhook_basic.send_text(
                msg=msg,
                con_info=con_info,
                at_ids=at_ids,
                at_mobiles=at_mobiles,
                is_at=is_at,
                at_all=at_all
            )
            if not ensure_success:
                return response
            else:
                if response.get('errcode') == 0:
                    return response
                else:
                    showlog.warning(response)
                    retry_count += 1
                    if retry_count > ensure_success_limit:
                        return response
                    time.sleep(1)
    elif webhook_hostname == 'open.feishu.cn':
        return lazyrequests.lazy_requests(
            url=webhook,
            method="POST",
            headers={"Content-Type": "application/json"},
            json={"msg_type":"text","content":{"text": msg}}
        )
    elif webhook_hostname == 'api.day.app':
        # 支持简单的Bark推送
        return lazyrequests.lazy_requests(
            url=webhook,
            method="GET"
        )
    else:
        return {'errcode': -2, 'errmsg': '暂不支持此webhook'}


def send(
        msg: str = None,

        env_file_name: str = None,
        webhook: str = None,
        webhooks: list = None,
        at_ids: list = None,
        at_mobiles: list = None,
        is_at: bool = None,
        at_all: bool = None,
        ensure_success: bool = False,
        ensure_success_limit: int = 60
):
    webhooks_all = list()
    send_res = list()
    if webhook:
        webhooks_all.append(webhook)
    if webhooks:
        webhooks_all.extend(webhooks)

    if not webhooks_all:
        showlog.warning("无有效的webhook地址")
    else:
        for each_index, each_webhook in enumerate(webhooks_all):
            showlog.info(f"正在发送第 {each_index+1}/{len(webhooks_all)} 条推送: \n{msg}\n")
            each_send_res = send_text(
                msg=msg,
                env_file_name=env_file_name,
                webhook=each_webhook,
                at_ids=at_ids,
                at_mobiles=at_mobiles,
                is_at=is_at,
                at_all=at_all,
                ensure_success=ensure_success,
                ensure_success_limit=ensure_success_limit
            )
            send_res.append({
                "webhook": each_webhook,
                "msg": msg,
                "send_res": each_send_res,
            })

    return send_res



#!/usr/bin/env python3
# coding = utf8
"""
@ Author : ZeroSeeker
@ e-mail : zeroseeker@foxmail.com
@ GitHub : https://github.com/ZeroSeeker
@ Gitee : https://gitee.com/ZeroSeeker
"""
import subprocess
import ipaddress
import requests
import json
import re


def get_public_ip() -> str:
    """
    获取当前网络公网ip地址
    备用地址：http://www.3322.org/dyndns/getip
    """
    import requests
    import json
    origin_ip = ''
    try:
        request_url = "http://httpbin.org/ip"
        response = requests.get(url=request_url)
        origin_ip = json.loads(response.text).get("origin")
    finally:
        return origin_ip


def get_local_ip() -> str:
    """
    获取内网ip地址
    """
    import socket
    ip = ''
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
    finally:
        s.close()
        return ip


def get_ip() -> dict:
    """
    获取当前网络ip地址（含有公网ip和内网ip）
    """
    origin_ip = get_public_ip()  # 获取公网ip
    local_ip = get_local_ip()  # 获取内网ip
    return {'origin_ip': origin_ip, 'local_ip': local_ip}


def get_ip_addr(ip: str):
    """
    查询ip归属地
    """
    api_url = f'http://whois.pconline.com.cn/ipJson.jsp?ip={ip}&json=true'
    response = requests.get(api_url)
    response_text = response.text.replace("\\", "-")
    addr = json.loads(response_text)['addr']
    return addr


def ipv6_exploded(addr: str):
    """
    将压缩的地址扩展
    """
    # 将压缩的IPv6地址字符串转换为IPv6地址对象
    ipv6_obj = ipaddress.IPv6Address(addr)
    # 使用 `.exploded` 属性获取完整的扩展形式
    return ipv6_obj.exploded


def ipv6_compressed(addr: str):
    """
    将标准地址压缩
    """
    # 将压缩的IPv6地址字符串转换为IPv6地址对象
    ipv6_obj = ipaddress.IPv6Address(addr)
    # 使用 `.compressed` 属性可以获取标准的压缩形式（去除前导零，压缩连续的零段）
    return ipv6_obj.compressed


def check_ipv6_exploded(ipv6_str):
    try:
        addr_obj = ipaddress.IPv6Address(ipv6_str)
        is_exploded = (addr_obj.exploded == ipv6_str)
        return is_exploded
    except ipaddress.AddressValueError:
        print(f"错误: '{ipv6_str}' 不是一个有效的IPv6地址。")
        return False


class JsonIP:
    """
    https://jsonip.com

    类似的服务还有：
    ident.me

    """
    def __init__(self):
        self.ip = get_ip()
        self.ipv4 = self.get_ipv4()
        self.ipv6 = self.get_ipv6()

    @staticmethod
    def get_ip():
        try:
            # 使用 ident.me 的API服务
            response = requests.get('https://jsonip.com', timeout=5)
            response.raise_for_status()  # 检查请求是否成功
            return response.json().get("ip")
        except requests.exceptions.RequestException as e:
            print(f"通过API[https://jsonip.com]获取公网IP地址失败: {e}")
            return None

    @staticmethod
    def get_ipv4():
        try:
            # 使用 ident.me 的API服务
            response = requests.get('https://ipv4.jsonip.com', timeout=5)
            response.raise_for_status()  # 检查请求是否成功
            return response.json().get("ip")
        except requests.exceptions.RequestException as e:
            print(f"通过API[https://jsonip.com]获取公网IPv4地址失败: {e}")
            return None

    @staticmethod
    def get_ipv6():
        try:
            # 使用 ident.me 的API服务
            response = requests.get('https://ipv6.jsonip.com', timeout=5)
            response.raise_for_status()  # 检查请求是否成功
            return response.json().get("ip")
        except requests.exceptions.RequestException as e:
            print(f"通过API[https://jsonip.com]获取公网IPv6地址失败: {e}")
            return None

    def get_public_ip_address(
            self,
            ip: bool = True,
            ipv4: bool = False,
            ipv6: bool = False
    ):
        res = dict()
        if ip:
            res["ip"] = self.get_ip()
        if ipv4:
            res["ipv4"] = self.get_ipv4()
        if ipv6:
            res["ipv6"] = self.get_ipv6()
        return res


class IPw:
    """
    https://ipw.cn/

    """
    def __init__(self):
        self.ip = get_ip()
        self.ipv4 = self.get_ipv4()
        self.ipv6 = self.get_ipv6()

    @staticmethod
    def get_ip():
        try:
            # 使用 ident.me 的API服务
            response = requests.get('https://test.ipw.cn/api/ip/myip?json', timeout=5)
            response.raise_for_status()  # 检查请求是否成功
            return response.json().get("IP")
        except requests.exceptions.RequestException as e:
            print(f"通过API[https://ipw.cn/]获取公网IP地址失败: {e}")
            return None

    @staticmethod
    def get_ipv4():
        try:
            # 使用 ident.me 的API服务
            response = requests.get('https://4.ipw.cn/api/ip/myip?json', timeout=5)
            response.raise_for_status()  # 检查请求是否成功
            return response.json().get("IP")
        except requests.exceptions.RequestException as e:
            print(f"通过API[https://4.ipw.cn]获取公网IPv4地址失败: {e}")
            return None

    @staticmethod
    def get_ipv6():
        try:
            # 使用 ident.me 的API服务
            response = requests.get('https://6.ipw.cn/api/ip/myip?json', timeout=5)
            response.raise_for_status()  # 检查请求是否成功
            return response.json().get("IP")
        except requests.exceptions.RequestException as e:
            print(f"通过API[https://6.ipw.cn]获取公网IPv6地址失败: {e}")
            return None

    def get_public_ip_address(
            self,
            ip: bool = True,
            ipv4: bool = False,
            ipv6: bool = False
    ):
        res = dict()
        if ip:
            res["ip"] = self.get_ip()
        if ipv4:
            res["ipv4"] = self.get_ipv4()
        if ipv6:
            res["ipv6"] = self.get_ipv6()
        return res


class IPAddr:
    def __init__(self):
        pass

    @staticmethod
    def get_ipv6_from_ip_command():
        """
        通过解析ip命令获取详细的IPv6信息
        """
        ipv6_details = []

        try:
            # 执行ip -6 addr show命令获取详细信息
            result = subprocess.run(['/usr/sbin/ip', '-6', 'addr', 'show'],
                                    capture_output=True, text=True, timeout=30)

            if result.returncode != 0:
                # 尝试使用ifconfig作为备选
                return get_ipv6_from_ifconfig()

            output = result.stdout
            lines = output.split('\n')

            current_interface = None
            current_addr_info = {}

            for line in lines:
                # 匹配接口行，如: "2: eth0: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500"
                interface_match = re.match(r'^\d+:\s+([^:]+):\s+<([^>]+)>', line)
                if interface_match:
                    if current_interface and current_addr_info:
                        ipv6_details.append(current_addr_info.copy())
                        current_addr_info = {}

                    current_interface = interface_match.group(1)
                    flags = interface_match.group(2).split(',')

                    current_addr_info = {
                        'interface': current_interface,
                        'flags': flags,
                        'ipv6_addresses': []
                    }

                    # 提取MTU值
                    mtu_match = re.search(r'mtu\s+(\d+)', line)
                    if mtu_match:
                        current_addr_info['mtu'] = int(mtu_match.group(1))

                # 匹配IPv6地址行
                if current_interface:
                    # 匹配格式: inet6 2001:db8::1/64 scope global
                    ipv6_match = re.search(r'inet6\s+([a-f0-9:]+)(?:%[^/]+)?/(\d+)\s+scope\s+(\w+)',
                                           line, re.IGNORECASE)
                    if ipv6_match:
                        ipv6_addr = ipv6_match.group(1)
                        prefix_len = ipv6_match.group(2)
                        scope = ipv6_match.group(3)

                        addr_details = {
                            'address': ipv6_addr,
                            'prefix_length': int(prefix_len),
                            'scope': scope,
                            'type': classify_ipv6_address(ipv6_addr)
                        }

                        # 提取更多标志
                        if 'dynamic' in line:
                            addr_details['dynamic'] = True
                        if 'noprefixroute' in line:
                            addr_details['noprefixroute'] = True
                        if 'valid_lft' in line:
                            # 提取有效期信息
                            valid_match = re.search(r'valid_lft\s+(\w+)\s+preferred_lft\s+(\w+)', line)
                            if valid_match:
                                addr_details['valid_lft'] = valid_match.group(1)
                                addr_details['preferred_lft'] = valid_match.group(2)

                        current_addr_info['ipv6_addresses'].append(addr_details)

            # 添加最后一个接口的信息
            if current_interface and current_addr_info:
                ipv6_details.append(current_addr_info)

        except Exception as e:
            print(f"解析ip命令输出时出错: {e}")
            return get_ipv6_from_ifconfig()

        return ipv6_details

    @staticmethod
    def get_ipv6_from_ifconfig():
        """
        备选方法：使用ifconfig命令
        """
        ipv6_details = []

        try:
            result = subprocess.run(['/usr/sbin/ifconfig'], capture_output=True, text=True, timeout=30)
            if result.returncode == 0:
                output = result.stdout
                interfaces = output.split('\n\n')

                for interface_block in interfaces:
                    lines = interface_block.split('\n')
                    if not lines or not lines[0]:
                        continue

                    # 提取接口名
                    interface_match = re.match(r'^(\S+)', lines[0])
                    if interface_match:
                        interface_name = interface_match.group(1)
                        interface_info = {
                            'interface': interface_name,
                            'ipv6_addresses': []
                        }

                        for line in lines:
                            # 匹配IPv6地址
                            ipv6_match = re.search(r'inet6\s+addr:\s*([a-f0-9:]+)', line, re.IGNORECASE)
                            if ipv6_match:
                                ipv6_addr = ipv6_match.group(1)
                                addr_details = {
                                    'address': ipv6_addr,
                                    'type': classify_ipv6_address(ipv6_addr)
                                }
                                interface_info['ipv6_addresses'].append(addr_details)

                        if interface_info['ipv6_addresses']:
                            ipv6_details.append(interface_info)

        except Exception as e:
            print(f"解析ifconfig输出时出错: {e}")

        return ipv6_details

    @staticmethod
    def classify_ipv6_address(ipv6_addr):
        """分类IPv6地址类型"""
        if ipv6_addr.startswith('fe80:'):
            return '链路本地地址'
        elif ipv6_addr.startswith('2000:'):
            return '全球单播地址'
        elif ipv6_addr.startswith('fc00:') or ipv6_addr.startswith('fd00:'):
            return '唯一本地地址'
        elif ipv6_addr == '::1':
            return '环回地址'
        elif ipv6_addr.startswith('ff00:'):
            return '组播地址'
        else:
            return '其他类型'

    @staticmethod
    def print_detailed_ipv6_info():
        """打印详细的IPv6信息"""
        ipv6_data = get_ipv6_from_ip_command()
        ipv6_real = None

        print("CentOS 7系统完整IPv6信息")
        print("=" * 100)

        if not ipv6_data:
            print("未找到IPv6地址信息")
            return

        for interface in ipv6_data:
            print(f"\n🔧 网络接口: {interface['interface']}")
            print(f"📊 接口标志: {', '.join(interface.get('flags', []))}")

            if 'mtu' in interface:
                print(f"📏 MTU: {interface['mtu']}")

            if interface['ipv6_addresses']:
                for i, addr in enumerate(interface['ipv6_addresses'], 1):
                    print(f"  📍 IPv6地址 #{i}:")
                    print(f"     地址: {addr['address']}")
                    print(f"     类型: {addr['type']}")
                    print(f"     前缀长度: /{addr.get('prefix_length', 'N/A')}")
                    print(f"     范围: {addr.get('scope', 'N/A')}")

                    address = addr['address']
                    if addr.get('scope', 'N/A') == "global" and addr.get('prefix_length', 'N/A') == 64:
                        ipv6_real = address

                    if 'dynamic' in addr:
                        print(f"     动态地址: 是")
                    if 'valid_lft' in addr:
                        print(f"     有效生存期: {addr.get('valid_lft', 'N/A')}")
                    if 'preferred_lft' in addr:
                        print(f"     首选生存期: {addr.get('preferred_lft', 'N/A')}")
            else:
                print("  该接口无IPv6地址")

            print("-" * 80)
        print("ipv6_real:", ipv6_real)
        return ipv6_real


if __name__ == '__main__':
    test = IPw()


#!/usr/bin/env python3
# coding = utf8
from lazysdk import showdata

def convert2dict(header_str: str):
    header_temp = header_str.split('\n')
    header_dict = dict()
    for each in header_temp:
        if len(each) == 0:
            continue
        else:
            each_split = each.split(': ')
            header_dict[each_split[0]] = each_split[1]
    return header_dict


def get_lines():
    lines = []
    print("请输入多行数据，以空行结束：")
    while True:
        line = input()
        if line:
            lines.append(line)
        else:
            break

    print("您输入的多行数据是：")
    return "\n".join(lines)


if __name__ == '__main__':
    test_lines = get_lines()
    showdata.show_dict(convert2dict(header_str=test_lines))




import platform
import os


def wifi():
    if platform.system() == 'Windows':
        wifi_cmd = ('netsh wlan show profiles ')
        with os.popen(wifi_cmd) as f:
            wifi_name_list = []
            for line in f:
                if '所有用户配置文件 :' in line:
                    # print(line)
                    line = line.strip()
                    wifi_name = line.split(':')[1]
                    wifi_name_list.append(wifi_name)
            print(wifi_name_list)
        for i in wifi_name_list:
            get = ('netsh wlan show profiles name={} key=clear').format(i)
            with os.popen(get) as r:
                print(r.read())
            r.close()
    else:
        pass



#!/usr/bin/env python3
# coding = utf8
from Crypto.Cipher import AES


def aes_decode(
        data,
        key
):
    """
    AES解密
    :param data:  要解密的数据
    :param key:  密钥（16.32）一般16的倍数
    :return:  处理好的数据
    """
    cryptor = AES.new(key, AES.MODE_CBC, key)
    plain_text = cryptor.decrypt(data)
    return plain_text.rstrip(b'\0')



from collections import OrderedDict


def make_thead(head_list: list):
    """
    制作表头，输入表头列表，输出生成的行代码列表
    """
    thead_list = list()
    for each_head in head_list:
        head_line = f'<th>{each_head}</th>'
        thead_list.append(head_line)
    return thead_list


def make_tbody(body_list: list):
    """
    制作表格主题，即表格的数据部分
    """
    tbody_list = list()
    for each_body in body_list:
        body_line = f'<td>{each_body}</td>'
        tbody_list.append(body_line)
    return tbody_list


def make_tb(
        data: list,
        border: int = 0,
        beautiful: bool = False,
        charset: str = "UTF-8",
        col_name_dict: OrderedDict = None
) -> str:
    """
    输入数据为list(dict())，输出生成的html表格代码
    border=2--> 边框宽度为2,是指外面灰色边框
    """
    key_list = list()
    for each_data in data:
        key_list.extend(list(each_data.keys()))
    key_list = list(set(key_list))
    if col_name_dict:
        key_list_new = list()
        for each_key in key_list:
            each_value = col_name_dict.get(each_key)
            if each_value:
                key_list_new.append(each_value)
            else:
                key_list_new.append(each_key)
        thead_list = make_thead(head_list=key_list_new)
    else:
        thead_list = make_thead(head_list=key_list)
    html_head = f"<tr>{''.join(thead_list)}</tr>"

    html_body = ''
    for each_data in data:
        body_list = list()
        for each_key in key_list:
            body_list.append(each_data.get(each_key))
        body_list = make_tbody(body_list=body_list)  # 这里每行数据都要生成一次
        html_body += f"<tr>{''.join(body_list)}</tr>\n"

    table_others = ''
    if border:
        table_others += f' border={border}'

    html_table = f"""<meta http-equiv="Content-Type" content="text/html;charset={charset}"/>\n<table{table_others}>\n{html_head}\n{html_body}</table>"""

    if beautiful:
        # 美化输出
        from lxml import html
        from lxml.html import builder as E
        # 创建一个lxml的Element对象
        root = html.fromstring(html_table)

        # 设置HTML的缩进和换行
        E.PRETTY_PRINT = True

        # 漂亮地打印HTML
        html_table = html.tostring(root, pretty_print=True)

    return html_table


def get_text(content: str):
    """
    从html中提取文本
    """
    import html2text
    return html2text.html2text(content)
